@@ -1,36 +1,48 @@
 #!/usr/bin/env python
 
+from functools import partial
 from random import randint
 from sqlalchemy import Column, func
 from sqlalchemy.ext.declarative import declarative_base, declared_attr 
-from sqlalchemy.orm import class_mapper, Query as _Query
+from sqlalchemy.orm import (backref as _backref, class_mapper, Query as _Query,
+  relationship as _relationship)
 from sqlalchemy.orm.attributes import InstrumentedAttribute
 from sqlalchemy.orm.collections import InstrumentedList
+from sqlalchemy.orm.exc import UnmappedClassError
 from sqlalchemy.orm.properties import ColumnProperty, RelationshipProperty
 
-from ..util import (Cacheable, _jsonify, JSONDepthExceededError,
-  JSONEncodedDict, Loggable, uncamelcase, query_to_dataframe)
+from ..util import (Cacheable, to_json, 
+  JSONEncodedDict, Loggable, uncamelcase, query_to_dataframe, query_to_records)
 
 
 class ORM(object):
 
-  config = {
-    'CREATE_ALL': True
-  }
+  """The main ORM extension.
 
-  def __init__(self, **kwargs):
-    for k, v in kwargs.items():
-      self.config[k.upper()] = v
+  Responsible for adding the ``q`` and ``c`` query proxies on the model and
+  configuring the project to use a custom ``Query`` class.
+
+  There is currently a single option available:
+
+  * ``CREATE_ALL`` to create tables for all models which don't already have one
+    (defaults to ``True``).
+
+  """
+
+  def __init__(self, project, create_all=True):
 
-  def on_register(self, project):
     project._query_class = Query
 
+    self.Model = declarative_base(cls=Base)
+    self.backref = partial(_backref, query_class=Query)
+    self.relationship = partial(_relationship, query_class=Query)
+
     @project.before_startup
     def handler(project):
-      if self.config['CREATE_ALL']:
-        Model.metadata.create_all(project._engine, checkfirst=True)
-      Model.q = _QueryProperty(project)
-      Model.c = _CountProperty(project)
+      self.Model.q = _QueryProperty(project)
+      self.Model.c = _CountProperty(project)
+      if create_all:
+        self.Model.metadata.create_all(project._engine, checkfirst=True)
 
 
 class Query(_Query):
@@ -42,14 +54,26 @@ class Query(_Query):
   """
 
   def get_or_404(self, model_id):
-    """Like get but aborts with 404 if not found."""
+    """Like get but aborts with 404 if not found.
+
+    :param model_id: the model's primary key
+    :type model_id: varies
+    :rtype: model or HTTPError
+    
+    """
     rv = self.get(model_id)
     if rv is None:
       abort(404)
     return rv
 
   def first_or_404(self):
-    """Like first but aborts with 404 if not found."""
+    """Like first but aborts with 404 if not found.
+    
+    :param model_id: the model's primary key
+    :type model_id: varies
+    :rtype: model or HTTPError
+    
+    """
     rv = self.first()
     if rv is None:
       abort(404)
@@ -83,7 +107,7 @@ class Query(_Query):
       return rv[0]
     return rv
 
-  def dataframe(self, *args, **kwargs):
+  def to_dataframe(self, *args, **kwargs):
     """Loads a dataframe with the records from the query and returns it.
 
     Accepts the same arguments as ``flasker.util.query_to_dataframe``.
@@ -92,9 +116,15 @@ class Query(_Query):
     """
     return query_to_dataframe(self, *args, **kwargs)
 
+  def to_records(self, *args, **kwargs):
+    """Raw execute of the query into a generator."""
+    return query_to_records(self, *args, **kwargs)
+
 
 class _QueryProperty(object):
 
+  """To make queries accessible directly on model classes."""
+
   def __init__(self, project):
     self.project = project
 
@@ -109,6 +139,8 @@ class _QueryProperty(object):
 
 class _CountProperty(object):
 
+  """To make count queries directly accessible on model classes."""
+
   def __init__(self, project):
     self.project = project
 
@@ -124,14 +156,25 @@ class _CountProperty(object):
 
 class Base(Cacheable, Loggable):
 
-  """Adding a few features to the declarative base.
+  """The SQLAlchemy base model with multiple helpers.
 
-  Currently:
+  Along with the methods described below, the following conveniences are
+  provided:
 
-  * Automatic table naming
-  * Caching
-  * Jsonifying
-  * Logging
+  * the ``q`` query property
+
+  * the ``c`` count query property. This will be much faster than issuing
+    ``count()`` on a normal query on MySQL as it will bypass the use of a
+    subquery.
+
+  * Automatic table naming (to the model's class name uncamelcased with an
+    extra s appended for good measure).
+
+  * ``__repr__`` implementation with model class and primary keys
+
+  * Caching (from ``flasker.util.Cacheable``)
+
+  * Logging (from ``flasker.util.Loggable``)
 
   """
 
@@ -141,16 +184,46 @@ class Base(Cacheable, Loggable):
   c = None
   q = None
 
-  def __init__(self, **kwargs):
-    for k, v in kwargs.items():
-      setattr(self, k, v)
+  @classmethod
+  def _get_columns(cls, show_private=False):
+    columns = class_mapper(cls).columns
+    if not show_private:
+      columns = [c for c in columns if not c.key.startswith('_')]
+    return columns
 
-  def __repr__(self):
-    primary_keys = ', '.join(
-      '%s=%r' % (k, getattr(self, k))
-      for k in self.__class__.get_primary_key_names()
-    )
-    return '<%s (%s)>' % (self.__class__.__name__, primary_keys)
+  @classmethod
+  def _get_related_models(cls, show_private=False):
+    return [
+      (r.key, r.mapper.class_)
+      for r in cls._get_relationships(show_private)
+    ]
+
+  @classmethod
+  def _get_relationships(cls, show_private=False):
+    rels =  class_mapper(cls).relationships.values()
+    if not show_private:
+      rels = [rel for rel in rels if not rel.key.startswith('_')]
+    return rels
+
+  @classmethod
+  def retrieve(cls, flush_if_new=True, **kwargs):
+    """Given constructor arguments will return a match or create one.
+
+    :param kwargs: constructor arguments
+    :rtype: tuple
+
+    This method returns a tuple ``(model, flag)`` where ``model`` is of the
+    corresponding class and ``flag`` is ``True`` if the model was just created
+    and ``False`` otherwise.
+
+    """
+    instance = cls.q.filter_by(**kwargs).first()
+    if instance:
+      return instance, False
+    instance = cls(**kwargs)
+    if flush_if_new:
+      instance.flush()
+    return instance, True
 
   @declared_attr
   def __json__(cls):
@@ -185,87 +258,63 @@ class Base(Cacheable, Loggable):
     """
     return '%ss' % uncamelcase(cls.__name__)
 
-  def __init__(self, **kwargs):
-    for k, v in kwargs.items():
-      setattr(self, k, v)
-
   def __repr__(self):
     primary_keys = ', '.join(
-      '%s=%r' % (k, v)
-      for k, v in self.get_primary_keys().items()
+      '%s=%r' % (k, getattr(self, k))
+      for k, v in self.get_primary_key().items()
     )
     return '<%s (%s)>' % (self.__class__.__name__, primary_keys)
 
-  def jsonify(self, depth=1, expand=True):
-    """Special implementation of jsonify for Model objects.
-    
-    Overrides the basic jsonify method to specialize it for models.
+  def get_primary_key(self):
+    """Returns the dictionary of primary keys for a given model.
 
-    This function minimizes the number of lookups it does (no dynamic
-    type checking on the properties for example) to maximize speed.
+    :rtype: dict
+
+    """
+    return dict(
+      (k.name, getattr(self, k.name))
+      for k in class_mapper(self.__class__).primary_key
+    )
+
+  def to_json(self, depth=1, expand=True):
+    """Special implementation of to_json for Model objects.
 
     :param depth:
     :type depth: int
+    :param expand: whether or not to repeat keys when jsonifying nested
+      objects (defaults to ``True``). This is useful when the frontend
+      has model support (e.g. using Backbone-Relational).
+    :type expand: bool
     :rtype: dict
 
+    The following attributes are included in the returned JSON:
+
+    * all non private properties
+    * all non private columns
+    * all non private relationships which have their ``lazy`` attribute set to
+      one of ``False, 'joined', 'immediate'``
+
+    The consequence of this is that this method will never issue extra queries
+    to populate the JSON. Furthermore, all the attribute names to be
+    included are computed at class declaration so this method is very fast.
+
     """
     if depth <= self._json_depth:
       # this instance has already been jsonified at a greater or
-      # equal depth, so we simply return its key (only used if expand is False)
-      return self.get_primary_keys()
+      # equal depth, so we simply return its key
+      return self.get_primary_key()
     if not expand:
       self._json_depth = depth
     rv = {}
     for varname in self.__json__:
       try:
-        rv[varname] = _jsonify(getattr(self, varname), depth - 1, expand)
+        rv[varname] = to_json(getattr(self, varname), depth - 1)
       except ValueError as e:
         rv[varname] = e.message
-      except JSONDepthExceededError:
-        pass
     return rv
 
-  def get_primary_keys(self):
-    return dict(
-      (k.name, getattr(self, k.name))
-      for k in self.__class__._primary_keys()
-    )
-
-  @classmethod
-  def find_or_create(cls, **kwargs):
-    instance = self.filter_by(**kwargs).first()
-    if instance:
-      return instance, False
-    instance = cls(**kwargs)
-    session = cls.q.db.session
-    session.add(instance)
-    session.flush()
-    return instance, True
-
-  @classmethod
-  def _columns(cls, show_private=False):
-    columns = class_mapper(cls).columns
-    if not show_private:
-      columns = [c for c in columns if not c.key.startswith('_')]
-    return columns
-
-  @classmethod
-  def _primary_keys(cls):
-    return class_mapper(cls).primary_key
-
-  @classmethod
-  def _related_models(cls, show_private=False):
-    return [
-      (r.key, r.mapper.class_)
-      for r in cls._relationships(show_private)
-    ]
-
-  @classmethod
-  def _relationships(cls, show_private=False):
-    rels =  class_mapper(cls).relationships.values()
-    if not show_private:
-      rels = [rel for rel in rels if not rel.key.startswith('_')]
-    return rels
-
-Model = declarative_base(cls=Base)
+  def flush(self):
+    session = self.q.session
+    session.add(self)
+    session.flush([self])
 
