@@ -15,13 +15,13 @@ from werkzeug.local import LocalProxy
 from .util import convert
 
 
-class _Tls(local):
+class _LocalStorage(local):
 
-  """Thread local storage for the project state."""
+  """Thread local storage."""
   
   _current_project = None
 
-_tls = _Tls()
+_local_storage = _LocalStorage()
 
 
 class ProjectImportError(Exception):
@@ -47,9 +47,9 @@ class Project(object):
       'DOMAIN': '',
       'SUBDOMAIN': '',
       'MODULES': '',
-      'APP_FOLDER': 'app',
-      'APP_STATIC_FOLDER': 'static',
-      'APP_TEMPLATE_FOLDER': 'templates',
+      'FLASK_ROOT_FOLDER': 'app',
+      'FLASK_STATIC_FOLDER': 'static',
+      'FLASK_TEMPLATE_FOLDER': 'templates',
       'COMMIT_ON_TEARDOWN': True,
     },
     'ENGINE': {
@@ -68,7 +68,7 @@ class Project(object):
   def __init__(self, config_path=None, make=True):
 
     self.__dict__ = self.__state
-    _tls._current_project = self
+    _local_storage._current_project = self
 
     if not self.__registered:
 
@@ -96,13 +96,12 @@ class Project(object):
 
       path.append(self.root_dir)
 
-      self.app = None
-      self.cel = None
+      self.flask = None
+      self.celery = None
       self.session = None
 
       self._engine = None
       self._query_class = Query
-      self._extensions = []
       self._before_startup = []
 
       self.__registered = True
@@ -113,10 +112,6 @@ class Project(object):
   def __repr__(self):
     return '<Project %r, %r>' % (self.config['PROJECT']['NAME'], self.root_dir)
 
-  def register_extension(self, extension, config_section=None):
-    """Register an extension."""
-    self._extensions.append((extension, config_section))
-
   def before_startup(self, func):
     """Decorator, hook to run a function right before project starts."""
     self._before_startup.append(func)
@@ -133,13 +128,6 @@ class Project(object):
     for mod in project_modules:
       __import__(mod.strip())
 
-    # extensions
-    for extension, config_section in self._extensions or []:
-      if config_section:
-        for k, v in self.config[config_section].items():
-          extension.config[k] = v
-      extension.on_register(self)
-
     # database
     self._setup_database_connection()
 
@@ -184,7 +172,7 @@ class Project(object):
         'Unable to parse configuration file at %s.' % config_path
       )
     conf = dict(
-      (s, dict((k, convert(v)) for (k, v) in parser.items(s)))
+      (s, dict((k, convert(v, allow_json=True)) for (k, v) in parser.items(s)))
       for s in parser.sections()
     )
     if not conf['PROJECT']['NAME']:
@@ -193,7 +181,7 @@ class Project(object):
 
 
 def _get_current_project():
-  return _tls._current_project or Project()
+  return _local_storage._current_project or Project()
 
 current_project = LocalProxy(_get_current_project)
 
